@@ -1,40 +1,54 @@
 use crate::commit::Commit;
 use crate::config::Config;
 use crate::error::Result;
+#[cfg(feature = "github")]
+use crate::github::{
+	GitHubClient,
+	GitHubCommit,
+	GitHubPullRequest,
+	FINISHED_FETCHING_MSG,
+	START_FETCHING_MSG,
+};
 use crate::release::{
 	Release,
 	Releases,
 };
 use crate::template::Template;
+use std::collections::HashMap;
 use std::io::Write;
+use std::time::{
+	SystemTime,
+	UNIX_EPOCH,
+};
 
 /// Changelog generator.
 #[derive(Debug)]
 pub struct Changelog<'a> {
-	releases: Vec<Release<'a>>,
-	template: Template,
-	config:   &'a Config,
+	releases:        Vec<Release<'a>>,
+	body_template:   Template,
+	footer_template: Option<Template>,
+	config:          &'a Config,
 }
 
 impl<'a> Changelog<'a> {
 	/// Constructs a new instance.
 	pub fn new(releases: Vec<Release<'a>>, config: &'a Config) -> Result<Self> {
-		let mut template = config
-			.changelog
-			.body
-			.as_deref()
-			.unwrap_or_default()
-			.to_string();
-		if config.changelog.trim.unwrap_or(true) {
-			template = template
-				.lines()
-				.map(|v| v.trim())
-				.collect::<Vec<&str>>()
-				.join("\n")
-		}
+		let trim = config.changelog.trim.unwrap_or(true);
 		let mut changelog = Self {
 			releases,
-			template: Template::new(template)?,
+			body_template: Template::new(
+				config
+					.changelog
+					.body
+					.as_deref()
+					.unwrap_or_default()
+					.to_string(),
+				trim,
+			)?,
+			footer_template: match &config.changelog.footer {
+				Some(footer) => Some(Template::new(footer.to_string(), trim)?),
+				None => None,
+			},
 			config,
 		};
 		changelog.process_commits();
@@ -135,25 +149,129 @@ impl<'a> Changelog<'a> {
 		}
 	}
 
+	/// Returns the GitHub metadata needed for the changelog.
+	///
+	/// This function creates a multithread async runtime for handling the
+	/// requests. The following are fetched from the GitHub REST API:
+	///
+	/// - Commits
+	/// - Pull requests
+	///
+	/// Each of these are paginated requests so they are being run in parallel
+	/// for speedup.
+	///
+	/// If no GitHub related variable is used in the template then this function
+	/// returns empty vectors.
+	#[cfg(feature = "github")]
+	fn get_github_metadata(
+		&self,
+	) -> Result<(Vec<GitHubCommit>, Vec<GitHubPullRequest>)> {
+		if self.body_template.contains_github_variable() ||
+			self.footer_template
+				.as_ref()
+				.map(|v| v.contains_github_variable())
+				.unwrap_or(false)
+		{
+			warn!("You are using an experimental feature! Please report bugs at <https://github.com/orhun/git-cliff/issues/new/choose>");
+			let github_client =
+				GitHubClient::try_from(self.config.remote.github.clone())?;
+			info!("{START_FETCHING_MSG} ({})", self.config.remote.github);
+			let data = tokio::runtime::Builder::new_multi_thread()
+				.enable_all()
+				.build()?
+				.block_on(async {
+					let (commits, pull_requests) = tokio::try_join!(
+						github_client.get_commits(),
+						github_client.get_pull_requests(),
+					)?;
+					debug!("Number of GitHub commits: {}", commits.len());
+					debug!("Number of GitHub pull requests: {}", commits.len());
+					Ok((commits, pull_requests))
+				});
+			info!("{FINISHED_FETCHING_MSG}");
+			data
+		} else {
+			Ok((vec![], vec![]))
+		}
+	}
+
+	/// Increments the version for the unreleased changes based on semver.
+	pub fn bump_version(&mut self) -> Result<Option<String>> {
+		if let Some(ref mut last_release) = self.releases.iter_mut().next() {
+			if last_release.version.is_none() {
+				let next_version = last_release.calculate_next_version()?;
+				debug!("Bumping the version to {next_version}");
+				last_release.version = Some(next_version.to_string());
+				last_release.timestamp = SystemTime::now()
+					.duration_since(UNIX_EPOCH)?
+					.as_secs()
+					.try_into()?;
+				return Ok(Some(next_version));
+			}
+		}
+		Ok(None)
+	}
+
 	/// Generates the changelog and writes it to the given output.
 	pub fn generate<W: Write>(&self, out: &mut W) -> Result<()> {
 		debug!("Generating changelog...");
+		let mut additional_context = HashMap::new();
+		additional_context.insert("remote", self.config.remote.clone());
+		#[cfg(feature = "github")]
+		let (github_commits, github_pull_requests) = self.get_github_metadata()?;
+		let postprocessors = self
+			.config
+			.changelog
+			.postprocessors
+			.clone()
+			.unwrap_or_default();
 		if let Some(header) = &self.config.changelog.header {
-			write!(out, "{header}")?;
-		}
-		for release in &self.releases {
-			let mut rendered = self.template.render(release)?;
-			if let Some(postprocessors) =
-				self.config.changelog.postprocessors.as_ref()
-			{
-				for postprocessor in postprocessors {
-					postprocessor.replace(&mut rendered, vec![])?;
+			let write_result = write!(out, "{header}");
+			if let Err(e) = write_result {
+				if e.kind() != std::io::ErrorKind::BrokenPipe {
+					return Err(e.into());
 				}
 			}
-			write!(out, "{}", rendered)?;
 		}
-		if let Some(footer) = &self.config.changelog.footer {
-			write!(out, "{footer}")?;
+		let mut releases = self.releases.clone();
+		for release in releases.iter_mut() {
+			#[cfg(feature = "github")]
+			release.update_github_metadata(
+				github_commits.clone(),
+				github_pull_requests.clone(),
+			)?;
+			let write_result = write!(
+				out,
+				"{}",
+				self.body_template.render(
+					&release,
+					Some(&additional_context),
+					&postprocessors
+				)?
+			);
+			if let Err(e) = write_result {
+				if e.kind() != std::io::ErrorKind::BrokenPipe {
+					return Err(e.into());
+				}
+			}
+		}
+		if let Some(footer_template) = &self.footer_template {
+			let write_result = writeln!(
+				out,
+				"{}",
+				footer_template.render(
+					&Releases {
+						releases: &releases,
+					},
+					Some(&additional_context),
+					&postprocessors,
+				)?
+			);
+			if let Err(e) = write_result {
+				if e.kind() != std::io::ErrorKind::BrokenPipe {
+					return Err(e.into());
+				}
+			}
 		}
 		Ok(())
 	}
@@ -175,7 +293,10 @@ impl<'a> Changelog<'a> {
 
 	/// Prints the changelog context to the given output.
 	pub fn write_context<W: Write>(&self, out: &mut W) -> Result<()> {
-		let output = Releases(&self.releases).as_json()?;
+		let output = Releases {
+			releases: &self.releases,
+		}
+		.as_json()?;
 		writeln!(out, "{output}")?;
 		Ok(())
 	}
@@ -188,6 +309,8 @@ mod test {
 		ChangelogConfig,
 		CommitParser,
 		GitConfig,
+		Remote,
+		RemoteConfig,
 		TextProcessor,
 	};
 	use pretty_assertions::assert_eq;
@@ -201,7 +324,7 @@ mod test {
 				body:           Some(String::from(
 					r#"{% if version %}
 				## Release [{{ version }}] - {{ timestamp | date(format="%Y-%m-%d") }}
-				({{ commit_id }}){% else %}
+				{% if commit_id %}({{ commit_id }}){% endif %}{% else %}
 				## Unreleased{% endif %}
 				{% for group, commits in commits | group_by(attribute="group") %}
 				### {{ group }}{% for group, commits in commits | group_by(attribute="scope") %}
@@ -209,7 +332,9 @@ mod test {
 				- {{ commit.message }}{% endfor %}
 				{% endfor %}{% endfor %}"#,
 				)),
-				footer:         Some(String::from("------------")),
+				footer:         Some(String::from(
+					r#"-- total releases: {{ releases | length }} --"#,
+				)),
 				trim:           Some(true),
 				postprocessors: Some(vec![TextProcessor {
 					pattern:         Regex::new("boring")
@@ -232,6 +357,40 @@ mod test {
 				}]),
 				commit_parsers:           Some(vec![
 					CommitParser {
+						sha:           Some(String::from("tea")),
+						message:       None,
+						body:          None,
+						group:         Some(String::from("I love tea")),
+						default_scope: None,
+						scope:         None,
+						skip:          None,
+						field:         None,
+						pattern:       None,
+					},
+					CommitParser {
+						sha:           Some(String::from("coffee")),
+						message:       None,
+						body:          None,
+						group:         None,
+						default_scope: None,
+						scope:         None,
+						skip:          Some(true),
+						field:         None,
+						pattern:       None,
+					},
+					CommitParser {
+						sha:           Some(String::from("coffee2")),
+						message:       None,
+						body:          None,
+						group:         None,
+						default_scope: None,
+						scope:         None,
+						skip:          Some(true),
+						field:         None,
+						pattern:       None,
+					},
+					CommitParser {
+						sha:           None,
 						message:       Regex::new(r".*merge.*").ok(),
 						body:          None,
 						group:         None,
@@ -242,6 +401,7 @@ mod test {
 						pattern:       None,
 					},
 					CommitParser {
+						sha:           None,
 						message:       Regex::new("feat*").ok(),
 						body:          None,
 						group:         Some(String::from("New features")),
@@ -252,6 +412,7 @@ mod test {
 						pattern:       None,
 					},
 					CommitParser {
+						sha:           None,
 						message:       Regex::new("^fix*").ok(),
 						body:          None,
 						group:         Some(String::from("Bug Fixes")),
@@ -262,6 +423,7 @@ mod test {
 						pattern:       None,
 					},
 					CommitParser {
+						sha:           None,
 						message:       Regex::new("doc:").ok(),
 						body:          None,
 						group:         Some(String::from("Documentation")),
@@ -272,6 +434,7 @@ mod test {
 						pattern:       None,
 					},
 					CommitParser {
+						sha:           None,
 						message:       Regex::new("docs:").ok(),
 						body:          None,
 						group:         Some(String::from("Documentation")),
@@ -282,6 +445,7 @@ mod test {
 						pattern:       None,
 					},
 					CommitParser {
+						sha:           None,
 						message:       Regex::new(r"match\((.*)\):.*").ok(),
 						body:          None,
 						group:         Some(String::from("Matched ($1)")),
@@ -292,6 +456,7 @@ mod test {
 						pattern:       None,
 					},
 					CommitParser {
+						sha:           None,
 						message:       Regex::new(".*").ok(),
 						body:          None,
 						group:         Some(String::from("Other")),
@@ -312,10 +477,25 @@ mod test {
 				link_parsers:             None,
 				limit_commits:            None,
 			},
+			remote:    RemoteConfig {
+				github: Remote {
+					owner: String::from("coolguy"),
+					repo:  String::from("awesome"),
+					token: None,
+				},
+			},
 		};
 		let test_release = Release {
-			version:   Some(String::from("v1.0.0")),
-			commits:   vec![
+			version: Some(String::from("v1.0.0")),
+			commits: vec![
+				Commit::new(
+					String::from("coffee"),
+					String::from("revert(app): skip this commit"),
+				),
+				Commit::new(
+					String::from("tea"),
+					String::from("feat(app): damn right"),
+				),
 				Commit::new(
 					String::from("0bc123"),
 					String::from("feat(app): add cool features"),
@@ -360,10 +540,18 @@ mod test {
 					String::from("qwert0"),
 					String::from("match(group): support regex-replace for groups"),
 				),
+				Commit::new(
+					String::from("coffee"),
+					String::from("revert(app): skip this commit"),
+				),
 			],
 			commit_id: Some(String::from("0bc123")),
 			timestamp: 50000000,
-			previous:  None,
+			previous: None,
+			#[cfg(feature = "github")]
+			github: crate::github::GitHubReleaseMetadata {
+				contributors: vec![],
+			},
 		};
 		let releases = vec![
 			test_release.clone(),
@@ -376,8 +564,8 @@ mod test {
 				..Release::default()
 			},
 			Release {
-				version:   None,
-				commits:   vec![
+				version: None,
+				commits: vec![
 					Commit::new(
 						String::from("abc123"),
 						String::from("feat(app): add xyz"),
@@ -399,10 +587,18 @@ mod test {
 						String::from("hjkl12"),
 						String::from("chore(ui): do boring stuff"),
 					),
+					Commit::new(
+						String::from("coffee2"),
+						String::from("revert(app): skip this commit"),
+					),
 				],
 				commit_id: None,
 				timestamp: 1000,
-				previous:  Some(Box::new(test_release)),
+				previous: Some(Box::new(test_release)),
+				#[cfg(feature = "github")]
+				github: crate::github::GitHubReleaseMetadata {
+					contributors: vec![],
+				},
 			},
 		];
 		(config, releases)
@@ -411,13 +607,16 @@ mod test {
 	#[test]
 	fn changelog_generator() -> Result<()> {
 		let (config, releases) = get_test_data();
-		let changelog = Changelog::new(releases, &config)?;
+		let mut changelog = Changelog::new(releases, &config)?;
+		changelog.bump_version()?;
+		changelog.releases[0].timestamp = 0;
 		let mut out = Vec::new();
 		changelog.generate(&mut out)?;
 		assert_eq!(
 			String::from(
 				r#"# Changelog
-			## Unreleased
+			## Release [v1.1.0] - 1970-01-01
+
 
 			### Bug Fixes
 			#### app
@@ -446,6 +645,10 @@ mod test {
 			- update docs
 			- add some documentation
 
+			### I love tea
+			#### app
+			- damn right
+
 			### Matched (group)
 			#### group
 			- support regex-replace for groups
@@ -468,7 +671,8 @@ mod test {
 
 			#### ui
 			- make good stuff
-			------------"#
+			-- total releases: 2 --
+			"#
 			)
 			.replace("			", ""),
 			str::from_utf8(&out).unwrap_or_default()
@@ -556,6 +760,10 @@ chore(deps): fix broken deps
 			- update docs
 			- add some documentation
 
+			### I love tea
+			#### app
+			- damn right
+
 			### Matched (group)
 			#### group
 			- support regex-replace for groups
@@ -583,7 +791,8 @@ chore(deps): fix broken deps
 
 			#### ui
 			- make good stuff
-			------------"#
+			-- total releases: 2 --
+			"#
 			)
 			.replace("			", ""),
 			str::from_utf8(&out).unwrap_or_default()
