@@ -8,6 +8,8 @@ use crate::error::{
 	Error as AppError,
 	Result,
 };
+#[cfg(feature = "github")]
+use crate::github::GitHubContributor;
 #[cfg(feature = "repo")]
 use git2::{
 	Commit as GitCommit,
@@ -120,6 +122,11 @@ pub struct Commit<'a> {
 	pub author:        Signature,
 	/// Committer.
 	pub committer:     Signature,
+	/// Whether if the commit has two or more parents.
+	pub merge_commit:  bool,
+	/// GitHub metadata of the commit.
+	#[cfg(feature = "github")]
+	pub github:        GitHubContributor,
 }
 
 impl<'a> From<String> for Commit<'a> {
@@ -152,6 +159,7 @@ impl<'a> From<&GitCommit<'a>> for Commit<'a> {
 			message: commit.message().unwrap_or_default().to_string(),
 			author: commit.author().into(),
 			committer: commit.committer().into(),
+			merge_commit: commit.parent_count() > 1,
 			..Default::default()
 		}
 	}
@@ -285,6 +293,21 @@ impl Commit<'_> {
 					})?,
 				));
 			}
+			if parser.sha.clone().map(|v| v.to_lowercase()).as_deref() ==
+				Some(&self.id)
+			{
+				if self.skip_commit(parser, protect_breaking) {
+					return Err(AppError::GroupError(String::from(
+						"Skipping commit",
+					)));
+				} else {
+					self.group = parser.group.clone().or(self.group);
+					self.scope = parser.scope.clone().or(self.scope);
+					self.default_scope =
+						parser.default_scope.clone().or(self.default_scope);
+					return Ok(self);
+				}
+			}
 			for (regex, text) in regex_checks {
 				if regex.is_match(&text) {
 					if self.skip_commit(parser, protect_breaking) {
@@ -292,16 +315,17 @@ impl Commit<'_> {
 							"Skipping commit",
 						)));
 					} else {
+						let regex_replace = |mut value: String| {
+							for mat in regex.find_iter(&text) {
+								value =
+									regex.replace(mat.as_str(), value).to_string();
+							}
+							value
+						};
 						self.group =
-							parser.group.as_ref().cloned().map(|mut group| {
-								for mat in regex.find_iter(&text) {
-									group = regex
-										.replace(mat.as_str(), group)
-										.to_string();
-								}
-								group
-							});
-						self.scope = parser.scope.as_ref().cloned();
+							parser.group.as_ref().cloned().map(regex_replace);
+						self.scope =
+							parser.scope.as_ref().cloned().map(regex_replace);
 						self.default_scope = parser.default_scope.as_ref().cloned();
 						return Ok(self);
 					}
@@ -413,6 +437,9 @@ impl Serialize for Commit<'_> {
 		commit.serialize_field("author", &self.author)?;
 		commit.serialize_field("committer", &self.committer)?;
 		commit.serialize_field("conventional", &self.conv.is_some())?;
+		commit.serialize_field("merge_commit", &self.merge_commit)?;
+		#[cfg(feature = "github")]
+		commit.serialize_field("github", &self.github)?;
 		commit.end()
 	}
 }
@@ -420,7 +447,6 @@ impl Serialize for Commit<'_> {
 #[cfg(test)]
 mod test {
 	use super::*;
-	use regex::Regex;
 	#[test]
 	fn conventional_commit() -> Result<()> {
 		let test_cases = vec![
@@ -441,6 +467,7 @@ mod test {
 		}
 		let commit = test_cases[0].0.clone().parse(
 			&[CommitParser {
+				sha:           None,
 				message:       Regex::new("test*").ok(),
 				body:          None,
 				group:         Some(String::from("test_group")),
@@ -613,6 +640,7 @@ mod test {
 
 		let parsed_commit = commit.parse(
 			&[CommitParser {
+				sha:           None,
 				message:       None,
 				body:          None,
 				group:         Some(String::from("Test group")),
@@ -629,4 +657,51 @@ mod test {
 		assert_eq!(Some(String::from("Test group")), parsed_commit.group);
 		Ok(())
 	}
+
+	#[test]
+	fn commit_sha() -> Result<()> {
+		let commit = Commit::new(
+			String::from("8f55e69eba6e6ce811ace32bd84cc82215673cb6"),
+			String::from("feat: do something"),
+		);
+		let parsed_commit = commit.clone().parse(
+			&[CommitParser {
+				sha:           Some(String::from(
+					"8f55e69eba6e6ce811ace32bd84cc82215673cb6",
+				)),
+				message:       None,
+				body:          None,
+				group:         None,
+				default_scope: None,
+				scope:         None,
+				skip:          Some(true),
+				field:         None,
+				pattern:       None,
+			}],
+			false,
+			false,
+		);
+		assert!(parsed_commit.is_err());
+
+		let parsed_commit = commit.parse(
+			&[CommitParser {
+				sha:           Some(String::from(
+					"8f55e69eba6e6ce811ace32bd84cc82215673cb6",
+				)),
+				message:       None,
+				body:          None,
+				group:         Some(String::from("Test group")),
+				default_scope: None,
+				scope:         None,
+				skip:          None,
+				field:         None,
+				pattern:       None,
+			}],
+			false,
+			false,
+		)?;
+		assert_eq!(Some(String::from("Test group")), parsed_commit.group);
+
+		Ok(())
+	}
 }
