@@ -7,6 +7,9 @@
 /// Command-line argument parser.
 pub mod args;
 
+/// Custom logger implementation.
+pub mod logger;
+
 #[macro_use]
 extern crate log;
 
@@ -18,21 +21,34 @@ use args::{
 use clap::ValueEnum;
 use git_cliff_core::changelog::Changelog;
 use git_cliff_core::commit::Commit;
-use git_cliff_core::config::Config;
-use git_cliff_core::embed::EmbeddedConfig;
+use git_cliff_core::config::{
+	CommitParser,
+	Config,
+};
+use git_cliff_core::embed::{
+	BuiltinConfig,
+	EmbeddedConfig,
+};
 use git_cliff_core::error::{
 	Error,
 	Result,
 };
 use git_cliff_core::release::Release;
 use git_cliff_core::repo::Repository;
-use git_cliff_core::DEFAULT_CONFIG;
+use git_cliff_core::{
+	DEFAULT_CONFIG,
+	IGNORE_FILE,
+};
+use secrecy::Secret;
 use std::env;
 use std::fs::{
 	self,
 	File,
 };
-use std::io;
+use std::io::{
+	self,
+	Write,
+};
 use std::time::{
 	SystemTime,
 	UNIX_EPOCH,
@@ -66,7 +82,7 @@ fn check_new_version() {
 /// repository individually.
 fn process_repository<'a>(
 	repository: &'static Repository,
-	config: Config,
+	config: &mut Config,
 	args: &Opt,
 ) -> Result<Vec<Release<'a>>> {
 	let mut tags = repository.tags(&config.git.tag_pattern, args.topo_order)?;
@@ -96,6 +112,19 @@ fn process_repository<'a>(
 		})
 		.collect();
 
+	if !config.remote.github.is_set() {
+		match repository.upstream_remote() {
+			Ok(remote) => {
+				debug!("No GitHub remote is set, using remote: {}", remote);
+				config.remote.github.owner = remote.owner;
+				config.remote.github.repo = remote.repo;
+			}
+			Err(e) => {
+				debug!("Failed to get remote from repository: {:?}", e);
+			}
+		}
+	}
+
 	// Print debug information about configuration and arguments.
 	log::trace!("{:#?}", args);
 	log::trace!("{:#?}", config);
@@ -177,8 +206,9 @@ fn process_repository<'a>(
 	let mut releases = vec![Release::default()];
 	let mut release_index = 0;
 	let mut previous_release = Release::default();
-	for git_commit in commits.into_iter().rev() {
-		let commit = Commit::from(&git_commit);
+	let mut first_processed_tag = None;
+	for git_commit in commits.iter().rev() {
+		let commit = Commit::from(git_commit);
 		let commit_id = commit.id.to_string();
 		if args.sort == Sort::Newest {
 			releases[release_index].commits.insert(0, commit);
@@ -196,6 +226,9 @@ fn process_repository<'a>(
 			} else {
 				git_commit.time().seconds()
 			};
+			if first_processed_tag.is_none() {
+				first_processed_tag = Some(tag);
+			}
 			previous_release.previous = None;
 			releases[release_index].previous = Some(Box::new(previous_release));
 			previous_release = releases[release_index].clone();
@@ -204,7 +237,7 @@ fn process_repository<'a>(
 		}
 	}
 
-	if release_index > 1 {
+	if release_index > 0 {
 		previous_release.previous = None;
 		releases[release_index].previous = Some(Box::new(previous_release));
 	}
@@ -220,32 +253,41 @@ fn process_repository<'a>(
 		}
 	}
 
-	// Set the previous release if needed.
-	if args.latest || args.unreleased {
-		let sub = if args.latest { 2 } else { 1 };
-		if let Some((commit_id, version)) =
-			tags.len().checked_sub(sub).and_then(|v| tags.get_index(v))
-		{
+	// Set the previous release if the first release does not have one set.
+	if !releases.is_empty() &&
+		releases
+			.first()
+			.and_then(|r| r.previous.as_ref())
+			.and_then(|p| p.version.as_ref())
+			.is_none()
+	{
+		// Get the previous tag of the first processed tag in the release loop.
+		let first_tag = first_processed_tag
+			.map(|tag| {
+				tags.iter()
+					.enumerate()
+					.find(|(_, (_, v))| v == &tag)
+					.and_then(|(i, _)| i.checked_sub(1))
+					.and_then(|i| tags.get_index(i))
+			})
+			.or_else(|| Some(tags.last()))
+			.flatten();
+
+		// Set the previous release if the first tag is found.
+		if let Some((commit_id, version)) = first_tag {
 			let previous_release = Release {
 				commit_id: Some(commit_id.to_string()),
 				version: Some(version.to_string()),
-				..Release::default()
+				timestamp: repository
+					.find_commit(commit_id.to_string())
+					.map(|v| v.time().seconds())
+					.unwrap_or_default(),
+				..Default::default()
 			};
 			releases[0].previous = Some(Box::new(previous_release));
 		}
 	}
 
-	// Bump the version.
-	if args.bump && releases[release_index].version.is_none() {
-		let next_version = releases[release_index].calculate_next_version()?;
-		debug!("Bumping the version to {next_version}");
-		releases[release_index].version = Some(next_version.to_string());
-		releases[release_index].timestamp = SystemTime::now()
-			.duration_since(UNIX_EPOCH)?
-			.as_secs()
-			.try_into()?;
-	}
-
 	Ok(releases)
 }
 
@@ -256,12 +298,24 @@ pub fn run(mut args: Opt) -> Result<()> {
 	check_new_version();
 
 	// Create the configuration file if init flag is given.
-	if args.init {
-		info!("Saving the configuration file to {:?}", DEFAULT_CONFIG);
-		fs::write(DEFAULT_CONFIG, EmbeddedConfig::get_config()?)?;
+	if let Some(init_config) = args.init {
+		let contents = match init_config {
+			Some(ref name) => BuiltinConfig::get_config(name.to_string())?,
+			None => EmbeddedConfig::get_config()?,
+		};
+		info!(
+			"Saving the configuration file{} to {:?}",
+			init_config.map(|v| format!(" ({v})")).unwrap_or_default(),
+			DEFAULT_CONFIG
+		);
+		fs::write(DEFAULT_CONFIG, contents)?;
 		return Ok(());
 	}
 
+	// Retrieve the built-in configuration.
+	let builtin_config =
+		BuiltinConfig::parse(args.config.to_string_lossy().to_string());
+
 	// Set the working directory.
 	if let Some(ref workdir) = args.workdir {
 		args.config = workdir.join(args.config);
@@ -281,7 +335,7 @@ pub fn run(mut args: Opt) -> Result<()> {
 	// Parse the configuration file.
 	let mut path = args.config.clone();
 	if !path.exists() {
-		if let Some(config_path) = dirs_next::config_dir()
+		if let Some(config_path) = dirs::config_dir()
 			.map(|dir| dir.join(env!("CARGO_PKG_NAME")).join(DEFAULT_CONFIG))
 		{
 			path = config_path;
@@ -289,7 +343,10 @@ pub fn run(mut args: Opt) -> Result<()> {
 	}
 
 	// Load the default configuration if necessary.
-	let mut config = if path.exists() {
+	let mut config = if let Ok((config, name)) = builtin_config {
+		info!("Using built-in configuration file: {name}");
+		config
+	} else if path.exists() {
 		Config::parse(&path)?
 	} else {
 		if !args.context {
@@ -341,22 +398,82 @@ pub fn run(mut args: Opt) -> Result<()> {
 			args.topo_order = topo_order;
 		}
 	}
+	if args.github_token.is_some() {
+		config.remote.github.token = args.github_token.clone().map(Secret::new);
+	}
+	if let Some(ref remote) = args.github_repo {
+		config.remote.github.owner = remote.0.owner.to_string();
+		config.remote.github.repo = remote.0.repo.to_string();
+	}
+	if args.no_exec {
+		if let Some(ref mut preprocessors) = config.git.commit_preprocessors {
+			preprocessors
+				.iter_mut()
+				.for_each(|v| v.replace_command = None);
+		}
+		if let Some(ref mut postprocessors) = config.changelog.postprocessors {
+			postprocessors
+				.iter_mut()
+				.for_each(|v| v.replace_command = None);
+		}
+	}
 	config.git.skip_tags = config.git.skip_tags.filter(|r| !r.as_str().is_empty());
 
-	// Process the repository.
+	// Process the repositories.
 	let repositories = args.repository.clone().unwrap_or(vec![env::current_dir()?]);
 	let mut releases = Vec::<Release>::new();
 	for repository in repositories {
+		// Skip commits
+		let mut skip_list = Vec::new();
+		let ignore_file = repository.join(IGNORE_FILE);
+		if ignore_file.exists() {
+			let contents = fs::read_to_string(ignore_file)?;
+			let commits = contents
+				.lines()
+				.filter(|v| !(v.starts_with('#') || v.trim().is_empty()))
+				.map(|v| String::from(v.trim()))
+				.collect::<Vec<String>>();
+			skip_list.extend(commits);
+		}
+		if let Some(ref skip_commit) = args.skip_commit {
+			skip_list.extend(skip_commit.clone());
+		}
+		if let Some(commit_parsers) = config.git.commit_parsers.as_mut() {
+			for sha1 in skip_list {
+				commit_parsers.insert(0, CommitParser {
+					sha: Some(sha1.to_string()),
+					skip: Some(true),
+					..Default::default()
+				})
+			}
+		}
+
+		// Process the repository.
 		let repository = Repository::init(repository)?;
 		releases.extend(process_repository(
 			Box::leak(Box::new(repository)),
-			config.clone(),
+			&mut config,
 			&args,
 		)?);
 	}
 
-	// Generate output.
-	let changelog = Changelog::new(releases, &config)?;
+	// Process commits and releases for the changelog.
+	let mut changelog = Changelog::new(releases, &config)?;
+
+	// Print the result.
+	if args.bump || args.bumped_version {
+		if let Some(next_version) = changelog.bump_version()? {
+			if args.bumped_version {
+				if let Some(path) = args.output {
+					let mut output = File::create(path)?;
+					output.write_all(next_version.as_bytes())?;
+				} else {
+					println!("{next_version}");
+				}
+				return Ok(());
+			}
+		}
+	}
 	if args.context {
 		return if let Some(path) = args.output {
 			let mut output = File::create(path)?;
